@@ -31,8 +31,13 @@ G_DECLARE_FINAL_TYPE (GrdRdpGfxFrameController, grd_rdp_gfx_frame_controller,
 
 GrdRdpGfxFrameController *grd_rdp_gfx_frame_controller_new (GrdRdpSurface *rdp_surface);
 
+GrdRdpGfxFramerateLog *grd_rdp_gfx_frame_controller_get_framerate_log (GrdRdpGfxFrameController *frame_controller);
+
+void grd_rdp_gfx_frame_controller_notify_history_changed (GrdRdpGfxFrameController *frame_controller);
+
 void grd_rdp_gfx_frame_controller_unack_frame (GrdRdpGfxFrameController *frame_controller,
                                                uint32_t                  frame_id,
+                                               uint32_t                  n_subframes,
                                                int64_t                   enc_time_us);
 
 void grd_rdp_gfx_frame_controller_ack_frame (GrdRdpGfxFrameController *frame_controller,
@@ -41,6 +46,7 @@ void grd_rdp_gfx_frame_controller_ack_frame (GrdRdpGfxFrameController *frame_con
 
 void grd_rdp_gfx_frame_controller_unack_last_acked_frame (GrdRdpGfxFrameController *frame_controller,
                                                           uint32_t                  frame_id,
+                                                          uint32_t                  n_subframes,
                                                           int64_t                   enc_ack_time_us);
 
 void grd_rdp_gfx_frame_controller_clear_all_unacked_frames (GrdRdpGfxFrameController *frame_controller);
